/*
 * Copyright 2008 Adam Tacy <adam.tacy AT gmail.com>
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package org.adamtacy.client.ui.effects;

import java.util.Iterator;
import java.util.Vector;

import org.adamtacy.client.ui.NEffectPanel;
import org.adamtacy.client.ui.effects.transitionsphysics.TransitionPhysics;

/**
 * Simple object to manage collection of effectHandlers.
 * 
 * @author tacyad
 * 
 */
@SuppressWarnings("serial")
public class NEffectPanelCollection extends Vector<NEffectPanel> {

  public void addEffect(NEffect e) {
    for (Iterator<NEffectPanel> it = iterator(); it.hasNext();)
      it.next().addEffect(e);      
  }
  

  public void addEffect(NEffect e, NEffectPanel p) {
    int index = indexOf(p);
    if (index < 0) throw new RuntimeException("Attempt to add effect to non-existent panel");
    elementAt(index).addEffect(e);    
  }

  public void cancelEffects() throws NEffectNotInterruptableException {
    for (Iterator<NEffectPanel> it = iterator(); it.hasNext();)
      it.next().cancelEffects();     
  }

  public void invertEffects() {
    for (Iterator<NEffectPanel> it = iterator(); it.hasNext();)
      it.next().invertEffects();     
  }

  public void playEffects() {
    for (Iterator<NEffectPanel> it = iterator(); it.hasNext();)
      it.next().playEffects();        
  }

  public void playEffects(double start, double end) {
    for (Iterator<NEffectPanel> it = iterator(); it.hasNext();)
      it.next().playEffects(start,end);     
  }

  public void removeEffect(NEffect theEffect) {
    for (Iterator<NEffectPanel> it = iterator(); it.hasNext();)
      it.next().removeEffect(theEffect);     
  }

  public void resumeEffectsBackwards() {
    for (Iterator<NEffectPanel> it = iterator(); it.hasNext();)
      it.next().resumeEffectsBackwards();     
  }

  public void resumeEffectsForwards() {
    for (Iterator<NEffectPanel> it = iterator(); it.hasNext();)
      it.next().resumeEffectsForwards();     
  }
  
  public void resumeEffectsBackwards(double end) {
    for (Iterator<NEffectPanel> it = iterator(); it.hasNext();)
      it.next().resumeEffectsBackwards(end);     
  }

  public void resumeEffectsForwards(double end) {
    for (Iterator<NEffectPanel> it = iterator(); it.hasNext();)
      it.next().resumeEffectsForwards(end);     
  }

  public void setEffectsLength(double duration) {
    for (Iterator<NEffectPanel> it = iterator(); it.hasNext();)
      it.next().setEffectsLength(duration);
  }

  public void setEffectsTransitionType(TransitionPhysics newTransition) {
    for (Iterator<NEffectPanel> it = iterator(); it.hasNext();)
      it.next().setEffectsTransitionType(newTransition);       
  }


  public void setPosition(double pos) {
    for (Iterator<NEffectPanel> it = iterator(); it.hasNext();)
      it.next().setPosition(pos);      
  }
}
