package org.adamtacy.client.ui.effects.core;

import org.adamtacy.client.ui.effects.impl.browsers.EffectImplementation;
import org.adamtacy.client.ui.effects.impl.css.StyleImplementation;

import com.google.gwt.dom.client.Element;

public class NChangeColorAction implements ChangeInterface {

  int currB;
  int currG;
  int currR;
  double currAlpha;

  int endB = 0;
  int endG = 255;
  int endR = 255;

  int startB = -1;
  int startG = -1;
  int startR = -1;
  double endAlpha = -1;
  double startAlpha = -1;

  public String getStartColour(){
    return "rgb("+startR+","+startG+","+startB+")";
  }
  
  public String getEndColour(){
    return "rgb("+endR+","+endG+","+endB+")";
  }
  
  public NChangeColorAction() {
  }

  public NChangeColorAction(String styleStart, String styleEnd) {
    this.setStartColor(styleStart);
    this.setEndColor(styleEnd);
  }

  public void performStep(Element effectElement, String styleComponentToChange,
      double progress) {
    double factor2 = progress;
    double factor1 = 1 - factor2;
    
    // Issue 43 to include handling transparent.
    // Note should use background-color as transparent too in the style
    if (isTransparent){
      EffectImplementation.setTransparent(effectElement, styleComponentToChange);
    } else {

    // Calculate new colour
    currR = (int) Math.floor(startR * factor1 + endR * factor2);
    currG = (int) Math.floor(startG * factor1 + endG * factor2);
    currB = (int) Math.floor(startB * factor1 + endB * factor2);

    StringBuffer colour = new StringBuffer();

    
    colour.append("(");
    colour.append(currR);
    colour.append(",");
    colour.append(currG);
    colour.append(",");
    colour.append(currB);
    
    if (startAlpha > -1){
      currAlpha = (endAlpha - startAlpha) * factor2;
      if (currAlpha < 0) currAlpha = -currAlpha;
      colour.append(",");
      colour.append(currAlpha);
      colour.insert(0,"rgba");
    } else colour.insert(0,"rgb");
    
    colour.append(")");
    
    
    EffectImplementation.setColour(effectElement, styleComponentToChange, colour.toString());
    }
  }

  public void reset() {
  }
  
  boolean isTransparent = false;



  /**
   * Change the start color to either a hex value or a rgb(r,g,b) value Both
   * contained within the input string
   * 
   * @param styleSheetColor
   */
  public void setEndColor(String styleSheetEndColor) {
    if (isTransparent) return;
    if (StyleImplementation.isTransparent(styleSheetEndColor)){
      isTransparent = true;
      return;
    }
    double[] startRGB = StyleImplementation.parseColour(styleSheetEndColor);
    endR = (int) startRGB[0];
    endG = (int) startRGB[1];
    endB = (int) startRGB[2];
    endAlpha = startRGB[3];
    // Align alpha value of start if we have one provided for end but not for start
    if (endAlpha>0 &&  startAlpha<0) startAlpha = 1;
  }

  /**
   * Change the start color to either a hex value or a rgb(r,g,b) value Both
   * contained within the input string
   * 
   * @param styleSheetColor
   */
  public void setStartColor(String styleSheetStartColor) {
    if (isTransparent) return;
    if (StyleImplementation.isTransparent(styleSheetStartColor)){
      isTransparent = true;
      return;
    }
    double[] startRGB = StyleImplementation.parseColour(styleSheetStartColor);
    startR = (int) startRGB[0];
    startG = (int) startRGB[1];
    startB = (int) startRGB[2];
    startAlpha = startRGB[3];
    // Align alpha value of end if we have one provided for start but not for end
    if (startAlpha>0 && endAlpha <0) endAlpha = 1;
  }
  
  String styleComponentToChange;
  String startColour;

  public void setUp(Element effectElement, String styleComponentToChange,
      double switchFrameNumber) {
    this.styleComponentToChange = styleComponentToChange;
    if (startR < 0) {
      // Haven't had the colour set for a start...so let's try and inherit it
      startColour = StyleImplementation.getComputedStyle(effectElement,
          styleComponentToChange);
      double[] rgb = StyleImplementation.parseColour(startColour);
      startR = (int) rgb[0];
      startG = (int) rgb[1];
      startB = (int) rgb[2];
      startAlpha = rgb[3];
    }
    this.performStep(effectElement, styleComponentToChange, 0.0);
  }

  public void tearDownEffect(Element effectElement) {
    EffectImplementation.setColour(effectElement, styleComponentToChange, startColour);
  }
}
