/*
 * Copyright 2008 Adam Tacy <adam.tacy AT gmail.com>
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package org.adamtacy.client.ui.effects.impl;

import java.util.Iterator;

import org.adamtacy.client.ui.effects.NEffect;
import org.adamtacy.client.ui.effects.SequentialCompositeEffect;
import org.adamtacy.client.ui.effects.impl.css.StyleImplementation;

import com.google.gwt.user.client.DOM;

/**
 * Highlight effect - sequences two Colour change events: a) From start colour
 * to end colour b) From end colour to start colour
 * 
 * As this extends the SequentialCompositeEffect class, it is possible to
 * request a delay between execution of the two colour changes to, for example,
 * keep the highlight for a period of time.
 * 
 */
public class Highlight extends SequentialCompositeEffect {

  /**
   * @deprecated
   * @author adam
   *
   */
  public class HighlightProperties extends BaseSequentialProperties {
    /**
     * Change the end color to either a hex value or a rgb(r,g,b) value Both
     * contained within the input string
     * 
     * @param styleSheetColor
     * @deprecated
     */
    public void setEndColor(String styleSheetEndColor) {
      for (Iterator<NEffect> it = internalEffects.iterator(); it.hasNext();) {
        NEffect theEffect = it.next();
        ((ChangeColor) theEffect).setEndColor(
            styleSheetEndColor);
      }
    }

    /**
     * Change the start color to either a hex value or a rgb(r,g,b) value Both
     * contained within the input string
     * 
     * @param styleSheetColor
     * @deprecated
     */
    public void setStartColor(String styleSheetStartColor) {
      for (Iterator<NEffect> it = internalEffects.iterator(); it.hasNext();) {
        NEffect theEffect = it.next();
        ((ChangeColor) theEffect).setStartColor(
            styleSheetStartColor);
      }
    }
  }
  
  /**
   * Change the end color to either a hex value or a rgb(r,g,b) value Both
   * contained within the input string
   * 
   * @param styleSheetColor
   */
  public void setEndColor(String styleSheetEndColor) {
    for (Iterator<NEffect> it = internalEffects.iterator(); it.hasNext();) {
      NEffect theEffect = it.next();
      ((ChangeColor) theEffect).setEndColor(
          styleSheetEndColor);
    }
  }

  /**
   * Change the start color to either a hex value or a rgb(r,g,b) value Both
   * contained within the input string
   * 
   * @param styleSheetColor
   */
  public void setStartColor(String styleSheetStartColor) {
    for (Iterator<NEffect> it = internalEffects.iterator(); it.hasNext();) {
      NEffect theEffect = it.next();
      ((ChangeColor) theEffect).setStartColor(
          styleSheetStartColor);
    }
  }

  /**
   * variable which holds original background colour before effect starts
   */
  String oldBackground;

  /**
   * Internal copy of properties inner class.
   * @deprecated
   */
  private HighlightProperties theHighlightProperties = new HighlightProperties();

  ChangeColor firstEffect;
  ChangeColor secondEffect;
  
  /**
   * Create the Highlight effect.
   * 
   */
  public Highlight() {
    super();

    // First Effect: Add a colour fade.
    firstEffect = new ChangeColor();
    registerEffect(firstEffect);

    // Second Effect: Add a colour fade that is inverted.
    secondEffect = new ChangeColor();
    secondEffect.invert();
    registerEffect(secondEffect);

    setEffectDelayLength(2);

    // Initialise the composite effect.
    this.initCompositeEffect();
  }

  /**
   * Finish composite effect; which restores the original background colour.
   */
  @Override
  public void beginCompositeEffect() {
    super.beginCompositeEffect();
    oldBackground = StyleImplementation.getComputedStyle(thePanel.getElement(),
        "backgroundColor");
    firstEffect.setStartColor(oldBackground);
  }

  /**
   * Finish composite effect; which restores the original background colour.
   */
  @Override
  public void finishCompositeEffect() {
    super.finishCompositeEffect();
    restoreColour();
  }

  /**
   * Provide ability for the user to get the properties object.
   * 
   * @return
   * @deprecated
   */
  public HighlightProperties getProperties() {
    return theHighlightProperties;
  }

  @Override
  protected void onUpdate(double progress) {
  }
  
  public void setPosition(double progress){
    double val = interpolate(progress);
    this.onUpdate(val);
  }

  private void restoreColour() {
    DOM.setStyleAttribute((com.google.gwt.user.client.Element) effectElement,
        "backgroundColor", oldBackground);
  }

  @Override
  public void setUpEffect() {
  }

  @Override
  public void tearDownEffect() {
    restoreColour();
  }
}
