package org.adamtacy.client.ui.unobtrusive;

import java.util.Iterator;

import org.adamtacy.client.ui.HasEffects;
import org.adamtacy.client.ui.NEffectPanel;
import org.adamtacy.client.ui.effects.NEffect;
import org.adamtacy.client.ui.effects.NEffectNotInterruptableException;
import org.adamtacy.client.ui.effects.NEffectPanelCollection;
import org.adamtacy.client.ui.effects.transitionsphysics.TransitionPhysics;

import com.google.gwt.user.client.ui.DockPanel;
import com.google.gwt.user.client.ui.HTMLTable;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Panel;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;
import com.google.gwt.user.client.ui.DockPanel.DockLayoutConstant;

/**
 * Class with two purposes:
 * 
 * a) To control several NEffectPanels at the same time (i.e. same effect on multiple widgets on various parts of screen)
 * b) Make the library less intrusive - with this class you can pass in a widget that already sits on a page and it will 
 * perform the necessary wrapping of that widget and replacing back in the screen.  Of course, multiple widgets can be 
 * wrapped and controlld at the same time.
 * 
 * @author adam
 * @deprecated New approach will be implemented in v5.0.0
 *
 */
public class EffectController implements HasEffects {

  private NEffectPanelCollection v;

  /**
   * Create a new EffectController using the widget w
   * @param w
   */
  public EffectController(Widget w) {
    this.add(w);
  }
  
  /**
   * Create a new EffectController using the widget w
   * @param w
   */
  public EffectController() {
  }

  /**
   * Add a new NEffectPanel to be controlled by the Controller
   * @param p new NEffectPanel 
   */
  public void add(NEffectPanel p) {
    if(v==null) v = new NEffectPanelCollection();
    v.add(p);
  }

  /**
   * Add a new widget to be controlled by the Controller (wraps widget with an NEffectPanel)
   * @param w new Widget to be controlled by the Controller.
   */
  public void add(Widget w) {
    if(v==null) v = new NEffectPanelCollection();
    v.add(createPanel(w));
  }

  /**
   * Adds an effect to all effect panels controlled by the Controller.
   */
  public void addEffect(NEffect e) {
    if(v!=null)v.addEffect(e);
  }

  /**
   * Cancel all efects controlled by this Controller.
   */
  public void cancelEffects() throws NEffectNotInterruptableException {
    if(v!=null)v.cancelEffects();
  }

  /**
   * Creates a new NEffectPanel for the passed in widget; widget must already be attached to the DOM structure
   * (if not, then you should use the normal creation of an NEffectPanel and add the widget to that).
   * @param w Widget that should be wrapped in an NEffectPanel
   * @return
   */
  protected NEffectPanel createPanel(Widget w) {
    assert (w.isAttached());
    Panel src = (Panel) w.getParent();
    // src.remove(w);
    NEffectPanel eP = new NEffectPanel();
    if (src instanceof VerticalPanel) {
      int currIndex = ((VerticalPanel) src).getWidgetIndex(w);
      ((VerticalPanel) src).insert(eP, currIndex);
    } else if (src instanceof HorizontalPanel) {
      int currIndex = ((HorizontalPanel) src).getWidgetIndex(w);
      ((HorizontalPanel) src).insert(eP, currIndex);
    } else if (src instanceof DockPanel) {
      DockLayoutConstant currIndex = ((DockPanel) src).getWidgetDirection(w);
      ((DockPanel) src).add(eP, currIndex);
    } else if (src instanceof HTMLTable) {
      int col = ((HTMLTable) src).getCellCount(0);
      // Find the widget index
      Iterator<Widget> i = ((HTMLTable) src).iterator();
      boolean found = false;
      int count = 0;
      while (i.hasNext() && !found){
        Widget compare = i.next();
        if (compare.equals(w)) found = true; 
        else count++;
      }
      // Turn index into x y co-ordinates
      
    }
    eP.add(w);
    return eP;
  }

  /**
   * Attempts to return the widget held by a panel, but since we have numerous panels in the controller, the 
   * result would make no sense; hence a RuntimeExcpetion is raised instead.
   */
  public Widget getPanelWidget() {
    throw new RuntimeException("Cannot return widget for an EffectController");
  }

  /**
   * Invert all the effects under the control of the Controller.
   */
  public void invertEffects() {
    v.invertEffects();
  }

  /**
   * Play all the effects under the control of the Controller from position 0.0 to position 1.0.
   */
  public void playEffects() {
    v.playEffects();
  }

  /**
   * Play all effects under control of the Controller between the start and end point.
   * @see NEffectPanel.playEffects(double start, double end) for how effects will be played.
   */
  public void playEffects(double start, double end) {
    v.playEffects(start, end);
  }

  /**
   * Remove an effect from all effect panels under control of the Controller.
   */
  public void removeEffect(NEffect theEffect) {
    v.removeEffect(theEffect);
  }

  /**
   * Resume playing all the effects attached to all effect panels under control of the Controller from their
   * current position to position 0.0
   */
  public void resumeEffectsBackwards() {
    v.resumeEffectsBackwards();
  }

  /**
   * Resume playing all the effects attached to all effect panels under control of the Controller from their
   * current position to position 1.0
   */
  public void resumeEffectsForwards() {
    v.resumeEffectsForwards();
  }

  /**
   * Set the effect duration of all effects attached to all effect panels under control of the Controller.
   */
  public void setEffectsLength(double duration) {
    v.setEffectsLength(duration);
  }

  /**
   * Set the transition type for all effects attached to all effect panels under control of the Controller.
   */
  public void setEffectsTransitionType(TransitionPhysics newTransition) {
    v.setEffectsTransitionType(newTransition);
  }
}
